defmodule Nulla.Models.Note do
  use Ecto.Schema
  import Ecto.Changeset
  import Ecto.Query
  alias Nulla.Repo
  alias Nulla.Models.Note

  schema "notes" do
    field :content, :string

    field :visibility, Ecto.Enum,
      values: [:public, :unlisted, :followers, :private],
      default: :public

    field :sensitive, :boolean, default: false
    field :language, :string
    field :in_reply_to, :string

    belongs_to :user, Nulla.Models.User
    has_many :media_attachments, Nulla.Models.MediaAttachment

    timestamps(type: :utc_datetime)
  end

  @doc false
  def changeset(note, attrs) do
    note
    |> cast(attrs, [:content, :visibility, :sensitive, :language, :in_reply_to, :user_id])
    |> validate_required([:content, :visibility, :sensitive, :language, :in_reply_to, :user_id])
  end

  def get_note!(id), do: Repo.get!(Note, id)

  def get_all_notes!(user_id), do: Repo.all(from n in Note, where: n.user_id == ^user_id)
end
