defmodule Nulla.Utils do
  import Ecto.Query
  alias Nulla.Repo
  alias Nulla.Models.User
  alias Nulla.Models.Follow
  alias Nulla.Models.InstanceSettings

  def count_following_by_username!(username) do
    case Repo.get_by(User, username: username) do
      nil ->
        {:error, :user_not_found}

      %User{id: user_id} ->
        count =
          Follow
          |> where([f], f.user_id == ^user_id)
          |> select([f], count(f.id))
          |> Repo.one()

        count
    end
  end

  def get_following_users_by_username!(username, page) when is_integer(page) and page > 0 do
    case Repo.get_by(User, username: username) do
      nil ->
        {:error, :user_not_found}

      %User{id: user_id} ->
        instance_settings = InstanceSettings.get_instance_settings!()
        per_page = instance_settings.offset
        offset = (page - 1) * per_page

        query =
          from [f, u] in
            from(f in Follow,
              join: u in User, on: u.id == f.target_id,
              where: f.user_id == ^user_id,
              order_by: [asc: u.inserted_at],
              offset: ^offset,
              limit: ^per_page,
              select: u
            )

        users = Repo.all(query)

        users
    end
  end

  def count_followers_by_username!(username) do
    case Repo.get_by(User, username: username) do
      nil ->
        0

      %User{id: user_id} ->
        from(f in Follow, where: f.target_id == ^user_id)
        |> select([f], count(f.id))
        |> Repo.one()
    end
  end

  def get_followers_by_username!(username, page) when is_integer(page) and page > 0 do
    case Repo.get_by(User, username: username) do
      nil ->
        {:error, :user_not_found}

      %User{id: user_id} ->
        instance_settings = InstanceSettings.get_instance_settings!()
        per_page = instance_settings.offset
        offset = (page - 1) * per_page

        query =
          from f in Follow,
            where: f.target_id == ^user_id,
            join: u in User, on: u.id == f.user_id,
            order_by: [asc: u.inserted_at],
            offset: ^offset,
            limit: ^per_page,
            select: u

        users = Repo.all(query)

        users
    end
  end
end
